// $Id$
/*****************************************************************************
 *   Copyright (C) 2010 by Bernd Flemisch                                    *
 *   Institute of Hydraulic Engineering                                      *
 *   University of Stuttgart, Germany                                        *
 *   email: <givenname>.<name>@iws.uni-stuttgart.de                          *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License, or       *
 *   (at your option) any later version, as long as this copyright notice    *
 *   is included in its original form.                                       *
 *                                                                           *
 *   This program is distributed WITHOUT ANY WARRANTY.                       *
 *****************************************************************************/
#ifndef DUMUX_1PSPATIALPARAMETERS_HH
#define DUMUX_1PSPATIALPARAMETERS_HH

#include <dumux/material/spatialparameters/boxspatialparameters.hh>

namespace Dumux
{

/** \todo Please doc me! */

template<class TypeTag>
class OnePSpatialParameters : public BoxSpatialParameters<TypeTag>
{
    typedef BoxSpatialParameters<TypeTag> ParentType;
    typedef typename GET_PROP_TYPE(TypeTag, PTAG(GridView)) GridView;
    typedef typename GET_PROP_TYPE(TypeTag, PTAG(Scalar)) Scalar;
    typedef typename GridView::ctype CoordScalar;

    enum {
        dimWorld=GridView::dimensionworld,
    };

    typedef typename GridView::template Codim<0>::Entity Element;
    typedef typename GET_PROP_TYPE(TypeTag, PTAG(FVElementGeometry)) FVElementGeometry;

public:

    OnePSpatialParameters(const GridView& gridView)
        : ParentType(gridView)
    {}

    /*!
     * \brief Apply the intrinsic permeability tensor to a pressure
     *        potential gradient.
     *
     * \param element       The current finite element
     * \param fvElemGeom    The current finite volume geometry of the element
     * \param scvIdx       The index sub-control volume face where the
     *                      intrinsic velocity ought to be calculated.
     */
    Scalar intrinsicPermeability(const Element           &element,
                                 const FVElementGeometry &fvElemGeom,
                                 int                      scvIdx) const
    {
        return 1e-10;
    }

    Scalar porosity(const Element           &element,
                    const FVElementGeometry &fvElemGeom,
                    int                      scvIdx) const
    {
        return 0.4;
    }

private:
};

} // end namespace
#endif

